package com.awp.homedirect.ejb.control;

import java.rmi.RemoteException;
import javax.ejb.*;
import java.util.Collection;

import com.awp.homedirect.ejb.profile.Profile;
import com.awp.homedirect.ejb.account.AccountHome;
import com.awp.homedirect.ejb.account.Account;
import com.awp.homedirect.ejb.account.AccountException;
import com.awp.homedirect.ejb.bill.BillHome;
import com.awp.homedirect.ejb.bill.Bill;
import com.awp.homedirect.ejb.transact.TransactHome;
import com.awp.homedirect.util.*;

public class ControlEJB implements javax.ejb.SessionBean
{
    // EJB context
    public javax.ejb.SessionContext EJB_Context;

    // Private fields
    private Profile profile = null;
    private AccountHome accountHome = null;
    private BillHome billHome = null;
    private TransactHome transactHome = null;

    // Lifecycle methods
    public void ejbCreate()
    {
        if(Trace.OK) System.out.println("HD<ControlEJB-ejbCreate>: Created");

        accountHome = LookupHome.getAccountHome();
        billHome = LookupHome.getBillHome();
        transactHome = LookupHome.getTransactHome();
    }

    public void ejbRemove()
    {
        if(Trace.OK) System.out.println("HD<ControlEJB-ejbRemove>: Removed");

        profile = null;
        accountHome = null;
        billHome = null;
        transactHome = null;
    }
    
    public void ejbActivate() {}
    public void ejbPassivate() {}

    public void setSessionContext(javax.ejb.SessionContext sc)
    { 
        EJB_Context = sc;
    }

    // Business methods
    public Profile getProfile()
    {
        return profile;
    }

    public void setProfile(Profile profile)
    {
        this.profile = profile;
    }

    public AccountHome getAccountHome()
    {
        return accountHome;
    }

    public BillHome getBillHome()
    {
        return billHome;
    }

    public TransactHome getTransactHome()
    {
        return transactHome;
    }

    // Method to return the collection of Accounts
    public Collection getAccounts()
    {
        Collection accounts = null;

        try
        {
            accounts = accountHome.findAccounts(profile.getCustomerid());
        }
        catch(RemoteException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-getAccounts>: Error 1 - " + e);
        }
        catch(FinderException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-getAccounts>: Error 2 - " + e);
        }

        return accounts;
    }

    // Method to return the collection of Transactions
    public Collection getTransacts()
    {
        Collection transacts = null;

        try
        {
            transacts = transactHome.findTransacts(profile.getCustomerid());
        }
        catch(RemoteException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-getTransacts>: Error 1 - " + e);
        }
        catch(FinderException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-getTransacts>: Error 2 - " + e);
        }

        return transacts;
    }

    // Method to return the collection of Unpaid Bills
    public Collection getUnpaidBills()
    {
        Collection bills = null;

        try
        {
            bills = billHome.findUnpaidBills(profile.getCustomerid());
        }
        catch(RemoteException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-getUnpaidBills>: Error 1 - " + e);
        }
        catch(FinderException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-getUnpaidBills>: Error 2 - " + e);
        }

        return bills;
    }

    // The guts of the Pay Bill transaction
    public String PayBill(String fromAccount,
                          String billId)
                  throws AccountException, GeneratorException
    {
        if(Trace.OK) System.out.println("HD<ControlEJB-PayBill>: Entry");

        String referenceId = null;

        try
        {
            // Get account instance
            Account fromAcct = accountHome.findByPrimaryKey(fromAccount);

            // Get bill instance and amount
            Bill bill = billHome.findByPrimaryKey(billId);
            long lAmount = bill.getDueamount();

            // Perform bill payment
            long lTotalAmount = fromAcct.withdrawl(lAmount);

            // Obtain reference id for the transaction
            referenceId = Generator.genRefid();

            // Build the explanation text for this payment
            String explanation = "Pay Bill - " + bill.getBillname() + " - ";
            if(lTotalAmount == lAmount)
            {
                explanation = explanation + "No fee";
            }
            else
            {
                explanation = explanation + "Fee: " + 
                              Convert.l2cur(lTotalAmount - lAmount);
            }

            // Create transaction record
            transactHome.create(profile.getCustomerid(),
                                explanation,
                                fromAccount,
                                "",
                                billId,
                                lAmount,
                                referenceId);

            // Mark bill as paid
            bill.setReferenceid(referenceId);
        }
        catch(GeneratorException e)
        {
            // Don't allow changes for the payment to be committed
            EJB_Context.setRollbackOnly();

            if(Trace.OK) System.out.println("HD<ControlEJB-PayBill>: Unable to get refid");

            // Rethrow the exception to the caller
            throw new GeneratorException();
        }
        catch(RemoteException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-PayBill>: Error 1 - " + e);
        }
        catch(FinderException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-PayBill>: Error 2 - " + e);
        }
        catch(CreateException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-PayBill>: Error 3 - " + e);
        }

        if(Trace.OK) System.out.println("HD<ControlEJB-PayBill>: Exit");

        return referenceId;
    }

    // The guts of the Transfer Funds transaction
    public String TransferFunds(String fromAccount,
                                String toAccount,
                                long lAmount)
                  throws AccountException, GeneratorException
    {
        if(Trace.OK) System.out.println("HD<ControlEJB-TransferFunds>: Entry");

        String referenceId = null;

        try
        {
            // Get account instances
            Account fromAcct = accountHome.findByPrimaryKey(fromAccount);
            Account toAcct = accountHome.findByPrimaryKey(toAccount);

            // Perform transfer of funds
            long lTotalAmount = fromAcct.withdrawl(lAmount);
            toAcct.deposit(lAmount);

            // Obtain reference id for the transaction
            referenceId = Generator.genRefid();

            // Build the explanation text for this transfer
            String explanation = "Transfer funds - ";
            if(lTotalAmount == lAmount)
            {
                explanation = explanation + "No fee";
            }
            else
            {
                explanation = explanation + "Fee: " + 
                              Convert.l2cur(lTotalAmount - lAmount);
            }

            // Create transaction record
            transactHome.create(profile.getCustomerid(),
                                explanation,
                                fromAccount,
                                toAccount,
                                "",
                                lAmount,
                                referenceId);
        }
        catch(GeneratorException e)
        {
            // Don't allow changes for the already transferred funds to be committed
            EJB_Context.setRollbackOnly();

            if(Trace.OK) System.out.println("HD<ControlEJB-TransferFunds>: Unable to get refid");

            // Rethrow the exception to the caller
            throw new GeneratorException();
        }
        catch(RemoteException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-TransferFunds>: Error 1 - " + e);
        }
        catch(FinderException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-TransferFunds>: Error 2 - " + e);
        }
        catch(CreateException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-TransferFunds>: Error 3 - " + e);
        }

        if(Trace.OK) System.out.println("HD<ControlEJB-TransferFunds>: Exit");

        return referenceId;
    }

    // Make updates to the profile
    public void updateProfileInfo(String salutation,
                                  String fullname,
                                  String password)
    {
        try
        {
            profile.setProfileInfo(salutation,
                                   fullname,
                                   password);
        }
        catch(RemoteException e)
        {
            if(Trace.OK) System.out.println("HD<ControlEJB-updateProfileInfo>: Error - " + e);
        }
    }
}
