package com.awp.homedirect.web;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import java.util.Collection;

import com.awp.homedirect.ejb.control.Control;
import com.awp.homedirect.ejb.account.AccountException;
import com.awp.homedirect.util.*;


public class BillServlet extends javax.servlet.http.HttpServlet
                         implements javax.servlet.SingleThreadModel
{
    public void doPost(javax.servlet.http.HttpServletRequest request,
                       javax.servlet.http.HttpServletResponse response)
                throws javax.servlet.ServletException, java.io.IOException 
    {
        if(Trace.OK) System.out.println("HD<BillServlet-doPost>: Entry");

        String fromAccount = request.getParameter("fromAccount");
        String billId = request.getParameter("billId");
        String paybill = request.getParameter("paybill");
        String password = request.getParameter("password");
        String redirect = "/Main.jsp";

        // Validate inputs
        if(paybill != null)
        {
            if((fromAccount == null) ||
               (billId == null))
            {
                if(Trace.OK) System.out.println("HD<BillServlet-doPost>: Invalid input");
                request.setAttribute("billText", TextData.FIXENTRYPAYBILLS);
                redirect = "/PayBills.jsp";
            }
            else
            {
                // If all inputs validated, prepare to pay bill
                HttpSession session = request.getSession(true);
                Control controlRemote = (Control)session.getAttribute("controlRemote"); 
                
                // If password checks out okay, then do the pay bill
                if(controlRemote.getProfile().getPassword().equals(password))
                {
                    try
                    {
                        String referenceId = controlRemote.PayBill(fromAccount,
                                                                   billId);
                        if(Trace.OK) System.out.println("HD<BillServlet-doPost>: Payment completed - refid: " +
                                                                                                  referenceId);
                        request.setAttribute("infoText", TextData.PAYBILLSCOMPLETED + " " + referenceId);

                        // Update number of unpaid bills
                        Collection bills = controlRemote.getUnpaidBills();
                        String unpaidBills = null;
                        if(bills.size() == 1)
                        {
                            unpaidBills = "<B>" + bills.size() + " bill</B>";
                        }
                        else
                        {
                            unpaidBills = "<B>" + bills.size() + " bills</B>";
                        }
                        session.setAttribute("unpaidBills", unpaidBills);
                    }
                    catch(AccountException e)
                    {
                        if(Trace.OK) System.out.println("HD<BillServlet-doPost>: Payment not allowed - " + e);
                        request.setAttribute("billText", TextData.PAYBILLSNOTALLOWED);
                        redirect = "/PayBills.jsp";
                    }
                    catch(GeneratorException e)
                    {
                        if(Trace.OK) System.out.println("HD<BillServlet-doPost>: Payment failed - rollback transaction");
                        request.setAttribute("billText", TextData.PAYBILLSFAILED);
                        redirect = "/PayBills.jsp";
                    }
                    catch(RemoteException e)
                    {
                        if(Trace.OK) System.out.println("HD<BillServlet-doPost>: Error 1 - " + e);
                    }
                }
                else
                {
                    if(Trace.OK) System.out.println("HD<BillServlet-doPost>: Invalid password");
                    request.setAttribute("billText", TextData.FIXENTRYPAYBILLS);
                    redirect = "/PayBills.jsp";
                }
            }
        }
        else
        {
            if(Trace.OK) System.out.println("HD<BillServlet-doPost>: Pay bills cancelled");
            request.setAttribute("infoText", TextData.CANCELPAYBILLS);
        }

        if(Trace.OK) System.out.println("HD<BillServlet-doPost>: Exit");

        // Redirect to the next JSP page
        RequestDispatcher dispatcher = getServletContext().getRequestDispatcher(redirect);
        dispatcher.forward(request, response);
    }
}
