package com.awp.homedirect.web;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.ejb.FinderException;
import javax.ejb.CreateException;
import java.util.Collection;

import com.awp.homedirect.util.*;
import com.awp.homedirect.ejb.profile.ProfileHome;
import com.awp.homedirect.ejb.profile.Profile;
import com.awp.homedirect.ejb.control.ControlHome;
import com.awp.homedirect.ejb.control.Control;

public class LoginServlet extends javax.servlet.http.HttpServlet
                          implements javax.servlet.SingleThreadModel
{
    public void doPost(javax.servlet.http.HttpServletRequest request,
                       javax.servlet.http.HttpServletResponse response)
                throws javax.servlet.ServletException, java.io.IOException 
    {
        if(Trace.OK) System.out.println("HD<LoginServlet-doPost>: Entry");

        String username = request.getParameter("username");
        String password = request.getParameter("password");
        String redirect = "/Login.jsp";

        // Validate username and password to allow login
        if((username == null) || (username.length() == 0) || 
           (password == null) || (password.length() == 0))
        {
            if(Trace.OK) System.out.println("HD<LoginServlet-doPost>: Empty user/pwd");
            request.setAttribute("loginErrorText", TextData.NEEDUSERPWD);
        }
        else
        {
            if(Trace.OK) System.out.println("HD<LoginServlet-doPost>: Scanning for " + username);

            try
            {
                // Check if profile exists before committing resources to this user
                ProfileHome profileHome = LookupHome.getProfileHome();
                Profile profileRemote = profileHome.findByPrimaryKey(username);

                if(Trace.OK) System.out.println("HD<LoginServlet-doPost>: Profile found");
		
                // Check for correct password		   
                if(profileRemote.getPassword().equals(password))
                {
                    try
                    {
                        // Now create the user session
                        ControlHome controlHome = LookupHome.getControlHome();
                        Control controlRemote = controlHome.create();

                        // Set the profile into the user session
                        controlRemote.setProfile(profileRemote);

                        // Set user session reference and customer name
                        HttpSession session = request.getSession(true);
                        session.setAttribute("controlRemote", controlRemote);
                        session.setAttribute("customerName", profileRemote.getCustomerName());

                        // Set number of unpaid bills
                        Collection bills = controlRemote.getUnpaidBills();
                        String unpaidBills = null;
                        if(bills.size() == 1)
                        {
                            unpaidBills = "<B>" + bills.size() + " bill</B>";
                        }
                        else
                        {
                            unpaidBills = "<B>" + bills.size() + " bills</B>";
                        }
                        session.setAttribute("unpaidBills", unpaidBills);
                       		     
                        // Set new redirect page
                        redirect = "/Main.jsp";

                        if(Trace.OK) System.out.println("HD<LoginServlet-doPost>: Valid login");
                    }
                    catch(CreateException e)
                    {
                        if(Trace.OK) System.out.println("HD<LoginServlet-doPost>: Error - " + e);
                    }
                }
                else
                {
                    if(Trace.OK) System.out.println("HD<LoginServlet-doPost>: Bad pwd");
                    request.setAttribute("loginErrorText", TextData.BADUSERPWD);
                }
            }
            catch(FinderException e)
            {
                if(Trace.OK) System.out.println("HD<LoginServlet-doPost>: Profile not found");
                request.setAttribute("loginErrorText", TextData.BADUSERPWD);
            }
        }

        if(Trace.OK) System.out.println("HD<LoginServlet-doPost>: Exit");

        // Redirect to the next JSP page
        RequestDispatcher dispatcher = getServletContext().getRequestDispatcher(redirect);
        dispatcher.forward(request, response);
    }
}
