package com.awp.homedirect.ejb.account;

import java.rmi.RemoteException;
import javax.ejb.*;
import java.util.Collection;

import com.awp.homedirect.util.*;
import com.awp.homedirect.ejb.account.AccountException;

public abstract class AccountEJB implements javax.ejb.EntityBean
{
    // EJB context
    public javax.ejb.EntityContext EJB_Context;

    // Private fields
    private boolean commitFlag = false;
    
    // Lifecycle methods
    public java.lang.String ejbCreate(String accountid,
                                      String accounttype,
                                      long balance,
                                      int customerid,
                                      long limit,
                                      long userfee)
                            throws javax.ejb.CreateException
    {
        setAccountid(accountid);
        setAccounttype(accounttype);
        setBalance(balance);
        setCustomerid(customerid);
        setLimit(limit);
        setUserfee(userfee);

        commitFlag = false;

        return null;
    }
    
    public void ejbPostCreate(String accountid,
                              String accounttype,
                              long balance,
                              int customerid,
                              long limit,
                              long userfee)
                throws javax.ejb.CreateException
    { 
    }
    
    public void ejbActivate() {}
    public void ejbPassivate() {}
 
    public void ejbLoad()
    { 
        if(Trace.OK) System.out.println("HD<AccountEJB-ejbLoad>: " + getAccountid());
        commitFlag = false;
    }
 
    public void ejbStore()
    {
        if(commitFlag)
        { 
            if(Trace.OK) System.out.println("HD<AccountEJB-ejbStore>: " + getAccountid() +
                                                                 " - THIS IS THE COMMIT");
            commitFlag = false;
        }
        else
        {
            if(Trace.OK) System.out.println("HD<AccountEJB-ejbStore>: " + getAccountid());
        }
    }
 
    public void ejbRemove() throws javax.ejb.RemoveException
    { 
    }
 
    public void setEntityContext(javax.ejb.EntityContext ctx)
    { 
        EJB_Context = ctx;
    }
 
    public void unsetEntityContext()
    { 
        EJB_Context = null;
    }

    // Accessor methods    
    public abstract String getAccountid();
    public abstract void setAccountid(String newAccountid);

    public abstract int getCustomerid();
    public abstract void setCustomerid(int newCustomerid);

    public abstract String getAccounttype();
    public abstract void setAccounttype(String newAccounttype);

    public abstract long getBalance();
    public abstract void setBalance(long newBalance);

    public abstract long getLimit();
    public abstract void setLimit(long newLimit);

    public abstract long getUserfee();
    public abstract void setUserfee(long newUserfee);

    // Business methods
    public String getBalanceStr()
    { 
        return Convert.l2cur(getBalance());
    }

    public String getLimitStr()
    { 
        // If this account represents a visa card
        if(getAccounttype().equals(TextData.VS_TYPE))
        {
            return Convert.l2cur(getLimit()) + " LIMIT";
        }
        else
        {
            if(getLimit() == 0)
            {
                return "NONE";
            }
            else
            {
                return Convert.l2cur(getLimit());
            }
        }
    }

    public String getUserfeeStr()
    { 
        if(getUserfee() == 0)
        {
            return "FREE";
        }
        else
        {
            return Convert.l2cur(getUserfee());
        }
    }

    public String getAccounttypeStr()
    { 
        String fullAccountType = null;
        String AcctType = getAccounttype();

        // Convert to full account type name
        if(AcctType.equals("PC"))
            fullAccountType = TextData.PC_TYPE;
        else if(AcctType.equals("PS"))
            fullAccountType = TextData.PS_TYPE;
        else if(AcctType.equals("SC"))
            fullAccountType = TextData.SC_TYPE;
        else if(AcctType.equals("SS"))
            fullAccountType = TextData.SS_TYPE;
        else if(AcctType.equals("VS"))
            fullAccountType = TextData.VS_TYPE;
       
        return fullAccountType;
    }

    public void setAccounttypeStr(String newAccounttype)
    { 
        // Convert from full account type name
        if(newAccounttype.equals(TextData.PC_TYPE))
            setAccounttype("PC");
        else if(newAccounttype.equals(TextData.PS_TYPE))
            setAccounttype("PS");
        else if(newAccounttype.equals(TextData.SC_TYPE))
            setAccounttype("SC");
        else if(newAccounttype.equals(TextData.SS_TYPE))
            setAccounttype("SS");
        else if(newAccounttype.equals(TextData.VS_TYPE))
            setAccounttype("VS");
    }
    
    public String getRowForBalances()
    {
        String sRow = "<TR align=\"center\">" +
                      "<TD bgcolor=white>" + getAccountid() +
                      "<TD bgcolor=white>" + getAccounttypeStr() +
                      "<TD bgcolor=white><B>" + getBalanceStr() + "</B>" +
                      "<TD bgcolor=white>" + getLimitStr() + "\n";

        return sRow;
    }

    public String getRowForPayBills()
    {
        String sRow = "<TR align=\"center\">" +
                      "<TD bgcolor=white align=\"left\"><INPUT type=\"radio\" name=\"fromAccount\" " +
                      "value=\"" + getAccountid() + "\">" + getAccountid() +
                      "<TD bgcolor=white>" + getAccounttypeStr() +
                      "<TD bgcolor=white><B>" + getBalanceStr() + "</B>" +
                      "<TD bgcolor=white>" + getLimitStr() + 
                      "<TD bgcolor=white>" + getUserfeeStr() + "\n";

        return sRow;
    }

    public String getRowForTransfer()
    {
        String sRow = "<TR align=\"center\">" +
                      "<TD bgcolor=white><INPUT type=\"radio\" name=\"fromAccount\" " +
                      "value=\"" + getAccountid() + "\">" +
                      "<TD bgcolor=white><INPUT type=\"radio\" name=\"toAccount\" " +
                      "value=\"" + getAccountid() + "\">" +
                      "<TD bgcolor=white>" + getAccountid() +
                      "<TD bgcolor=white>" + getAccounttypeStr() +
                      "<TD bgcolor=white><B>" + getBalanceStr() + "</B>" +
                      "<TD bgcolor=white>" + getLimitStr() + 
                      "<TD bgcolor=white>" + getUserfeeStr() + "\n";

        return sRow;
    }

    public void deposit(long lAmount)
    {

        // If this account represents a visa card
        if(getAccounttypeStr().equals(TextData.VS_TYPE))
        {
            setBalance(getBalance() - lAmount);
        }
        else
        {
            // For normal bank accounts
            setBalance(getBalance() + lAmount);
        }

        // Next ejbStore represents the COMMIT of these changes
        commitFlag = true;
    }

    public long withdrawl(long lAmount) throws AccountException
    {
        long lTotalAmount = lAmount + getUserfee();
     
        // If this account represents a visa card
        if(getAccounttypeStr().equals(TextData.VS_TYPE))
        {
            if((getBalance() + lTotalAmount) <= getLimit())
            {
                setBalance(getBalance() + lTotalAmount);
            }
            else
            {
                if(Trace.OK) System.out.println("HD<AccountEJB-withdrawl>: " + getAccountid() +
                                                                      " - Visa limit reached");
                throw new AccountException("Visa limit reached for " + getAccountid());
            }
        }
        else
        {
            // For normal bank accounts
            if((getBalance() - lTotalAmount) >= (getLimit() * -1))
            {
                setBalance(getBalance() - lTotalAmount);
            }
            else
            {
                if(Trace.OK) System.out.println("HD<AccountEJB-withdrawl>: " + getAccountid() +
                                                                      " - Insufficient funds");
                throw new AccountException("Insufficient funds for " + getAccountid());
            }
        }

        // Next ejbStore represents the COMMIT of these changes
        commitFlag = true;

        // Return the total amount withdrawn
        return lTotalAmount;
    }
}
