package com.awp.homedirect.util;

import java.util.GregorianCalendar;
import java.util.Calendar;
import java.util.Date;
import com.awp.homedirect.util.GeneratorException;

public final class Generator
{
    //********************************************************************************
    // Generates a 16 character reference id based on the current date and time.
    // This method also throws an exception if the 'seconds' portion of the current
    // time is between 0 and 5. This method is intended to simulate a reference id
    // being generated by a remote system, which fails occasionally. This allows us
    // to better illustrate what happens when transactions involving EJBs need to
    // be rolled back due to uncontrollable failures.
    //********************************************************************************
    public static String genRefid() throws GeneratorException
    {
        String sOut = null;
        String sTemp = null;

        // Set up current date and time
        Calendar curCal = new GregorianCalendar();
        Date now = new Date();
        curCal.setTime(now);

        // If the current seconds is 5 or less, throw an exception,
        // imitating a failed call to a remote system
        int iSec = curCal.get(Calendar.SECOND);
        if(iSec <= 5)
        {
            throw new GeneratorException();
        }

        // Generate 16 character reference id
        sOut = String.valueOf(curCal.get(Calendar.YEAR));
     
        sTemp = String.valueOf(curCal.get(Calendar.MONTH));
        if(sTemp.length() < 2)
        {
            sTemp = "0" + sTemp;
        }
        sOut = sOut + sTemp;

        sTemp = String.valueOf(curCal.get(Calendar.DATE));
        if(sTemp.length() < 2)
        {
            sTemp = "0" + sTemp;
        }
        sOut = sOut + sTemp;

        sTemp = String.valueOf(curCal.get(Calendar.HOUR_OF_DAY));
        if(sTemp.length() < 2)
        {
            sTemp = "0" + sTemp;
        }
        sOut = sOut + sTemp;

        sTemp = String.valueOf(curCal.get(Calendar.MINUTE));
        if(sTemp.length() < 2)
        {
            sTemp = "0" + sTemp;
        }
        sOut = sOut + sTemp;

        sTemp = String.valueOf(iSec);
        if(sTemp.length() < 2)
        {
            sTemp = "0" + sTemp;
        }
        sOut = sOut + sTemp;

        sTemp = String.valueOf(curCal.get(Calendar.MILLISECOND));
        if(sTemp.length() < 2)
        {
            sTemp = "0" + sTemp;
        }
        sOut = sOut + sTemp.substring(0,2);

        return sOut;
    }
}
