package com.awp.homedirect.web;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.rmi.RemoteException;
import javax.ejb.RemoveException;
import java.util.Collection;

import com.awp.homedirect.util.*;
import com.awp.homedirect.ejb.control.Control;
import com.awp.homedirect.ejb.profile.Profile;
import com.awp.homedirect.web.ProfileInfo;
import com.awp.homedirect.web.AccountInfo;
import com.awp.homedirect.web.TransactInfo;

public class MainServlet extends javax.servlet.http.HttpServlet
                         implements javax.servlet.SingleThreadModel
{
    public void doGet(javax.servlet.http.HttpServletRequest request,
                      javax.servlet.http.HttpServletResponse response) 
                      throws javax.servlet.ServletException, java.io.IOException 
    {
        if(Trace.OK) System.out.println("HD<MainServlet-doGet>: Entry");

        String action = request.getParameter("action_id");
        String redirect = "/Login.jsp";

        // Validate action first
        if((action == null) || (action.length() == 0))
        {
            if(Trace.OK) System.out.println("HD<MainServlet-doGet>: Error 1");
        }
        else
        {
            if(Trace.OK) System.out.println("HD<MainServlet-doGet>: Action is " + action);

            try
            {
                // Get the user session and profile instances
                HttpSession session = request.getSession(true);
                Control controlRemote = (Control)session.getAttribute("controlRemote");
                
                if(action.equals("BALANCE") || action.equals("TRANSFER")) 
                {
                    // Find collection of accounts
                    Collection accounts = controlRemote.getAccounts();
 
                    // Create bean to pass info to JSP page
                    AccountInfo accountInfo = new AccountInfo(accounts);
                    session.setAttribute("AccountInfo", accountInfo);
                    
                    if(action.equals("BALANCE"))
                    {
                        redirect = "/Balances.jsp";
                    }
                    else
                    {
                        redirect = "/Transfer.jsp";
                    }
                }
                else if(action.equals("PAYBILLS"))
                {
                    // Find collection of accounts
                    Collection accounts = controlRemote.getAccounts();


                    // Find collection of unpaid bills
                    Collection bills = controlRemote.getUnpaidBills();
 
                    // Create bean to pass account and bill info to JSP page
                    AccountInfo accountInfo = new AccountInfo(accounts, bills);
                    session.setAttribute("AccountInfo", accountInfo);
                    redirect = "/PayBills.jsp";
                }
                else if(action.equals("TRANSACT"))
                {
                    // Find collection of transacts
                    Collection transacts = controlRemote.getTransacts();
 
                    // Create bean to pass info to JSP page
                    TransactInfo transactInfo = new TransactInfo(transacts);
                    session.setAttribute("TransactInfo", transactInfo);
                    redirect = "/ListTransacts.jsp";
                }
                else if(action.equals("PROFILE"))
                {
                    // Create bean to pass profile info to jsp page
                    ProfileInfo profileInfo = new ProfileInfo(controlRemote.getProfile());
                    session.setAttribute("ProfileInfo", profileInfo);
                    redirect = "/EditProfile.jsp";
                }
                else
                {
                    try
                    {
                        // Remove user session and clear the references 
                        controlRemote.remove();
                        session.setAttribute("controlRemote", "");
                        session.setAttribute("customerName", "");
                        session.setAttribute("unpaidBills", "");

                        if(Trace.OK) System.out.println("HD<MainServlet-doGet>: Session removed");
                    }
                    catch(RemoveException e)
                    {
                        if(Trace.OK) System.out.println("HD<MainServlet-doGet>: Error 2 - " + e);
                    }

                    // Set logoff page for redirect
                    redirect = "/Logoff.jsp";
                }
            }
            catch(RemoteException e)
            {
                if(Trace.OK) System.out.println("HD<MainServlet-doGet>: Not logged in");
                request.setAttribute("loginErrorText", TextData.NOTLOGGEDIN);
            }
        }

        if(Trace.OK) System.out.println("HD<MainServlet-doGet>: Exit");

        // Redirect to next JSP page
        RequestDispatcher dispatcher = getServletContext().getRequestDispatcher(redirect);
        dispatcher.forward(request, response);
    }
}
