package com.awp.homedirect.web;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.rmi.RemoteException;

import com.awp.homedirect.ejb.control.Control;
import com.awp.homedirect.ejb.account.AccountException;
import com.awp.homedirect.util.*;


public class TransferServlet extends javax.servlet.http.HttpServlet
                             implements javax.servlet.SingleThreadModel
{
    public void doPost(javax.servlet.http.HttpServletRequest request,
                       javax.servlet.http.HttpServletResponse response)
                throws javax.servlet.ServletException, java.io.IOException 
    {
        if(Trace.OK) System.out.println("HD<TransferServlet-doPost>: Entry");

        String fromAccount = request.getParameter("fromAccount");
        String toAccount = request.getParameter("toAccount");
        String amount = request.getParameter("amount");
        String transfer = request.getParameter("transfer");
        String password = request.getParameter("password");
        String redirect = "/Main.jsp";

        // Validate inputs
        if(transfer != null)
        {
            long lAmount = 0;

            if((amount != null) && (amount.length() > 0))
            {
                try
                {
                    Double dAmount = new Double(amount);
 
                    if(dAmount != null)
                    {
                        dAmount = new Double(dAmount.doubleValue() * 100);
                        lAmount = dAmount.longValue();
                    }
                }
                catch(NumberFormatException e)
                {
                    if(Trace.OK) System.out.println("HD<TransferServlet-doPost>: Bad input - " + e);
                }
            }

            if((fromAccount == null) ||
               (toAccount == null) ||
               (lAmount <= 0))
            {
                if(Trace.OK) System.out.println("HD<TransferServlet-doPost>: Invalid input");
                request.setAttribute("transferText", TextData.FIXENTRYTRANSFER);
                redirect = "/Transfer.jsp";
            }
            else
            {
                // If all inputs validated, prepare to transfer
                HttpSession session = request.getSession(true);
                Control controlRemote = (Control)session.getAttribute("controlRemote"); 
                
                // If password checks out okay, then do the transfer
                if(controlRemote.getProfile().getPassword().equals(password))
                {
                    try
                    {
                        String referenceId = controlRemote.TransferFunds(fromAccount,
                                                                         toAccount,
                                                                         lAmount);
                        if(Trace.OK) System.out.println("HD<TransferServlet-doPost>: Transfer completed - refid: " +
                                                                                                       referenceId);
                        request.setAttribute("infoText", TextData.TRANSFERCOMPLETED + " " + referenceId);
                    }
                    catch(AccountException e)
                    {
                        if(Trace.OK) System.out.println("HD<TransferServlet-doPost>: Transfer not allowed - " + e);
                        request.setAttribute("transferText", TextData.TRANSFERNOTALLOWED);
                        redirect = "/Transfer.jsp";
                    }
                    catch(GeneratorException e)
                    {
                        if(Trace.OK) System.out.println("HD<TransferServlet-doPost>: Transfer failed - rollback transaction");
                        request.setAttribute("transferText", TextData.TRANSFERFAILED);
                        redirect = "/Transfer.jsp";
                    }
                    catch(RemoteException e)
                    {
                        if(Trace.OK) System.out.println("HD<TransferServlet-doPost>: Error - " + e);
                    }
                }
                else
                {
                    if(Trace.OK) System.out.println("HD<TransferServlet-doPost>: Invalid password");
                    request.setAttribute("transferText", TextData.FIXENTRYTRANSFER);
                    redirect = "/Transfer.jsp";
                }
            }
        }
        else
        {
            if(Trace.OK) System.out.println("HD<TransferServlet-doPost>: Transfer cancelled");
            request.setAttribute("infoText", TextData.CANCELTRANSFER);
        }

        if(Trace.OK) System.out.println("HD<TransferServlet-doPost>: Exit");

        // Redirect to the next JSP page
        RequestDispatcher dispatcher = getServletContext().getRequestDispatcher(redirect);
        dispatcher.forward(request, response);
    }
}
