import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.InputStreamReader;
import java.io.File;
import java.io.UnsupportedEncodingException;
import java.sql.SQLException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.DriverManager;
import java.util.StringTokenizer;
import java.util.Vector;

// Reads a script file and executes the SQL statements against the Cloudscape DB
public class Resetdb
{
    public static void main(String[] args)
    {
        if(args.length > 0)
        {
            System.out.println("\n*********************************************************************");
            System.out.println("Resetdb - Reads a script file to reset Cloudscape DB tables and data.");
            System.out.println("*********************************************************************\n");
            resetdb(args[0], args[1]);
            System.out.println("**** Resetdb completed succcessfully. ****");
        }
        else
        {
            System.out.println("Missing arguments - correct usage is: Resetdb <script_file> <host_name>");
        }
    }

    public static void resetdb(String fileName, String hostName)
    {
        String dbDriver = "RmiJdbc.RJDriver";
        String dbName = "jdbc:rmi://" + hostName +
                        ":1099/jdbc:cloudscape:HomeDirectDB;autocommit=true;create=true";

        System.out.println("Connecting to host: " + hostName);

        Connection conn = null;

        // Check driver
        try
        {
            Class.forName(dbDriver);
        }
        catch(ClassNotFoundException e)
	{
            System.out.println("DB driver error: " + e + "\n");
            System.exit(0);
        }

        // Get connection
        try
        {
            conn = DriverManager.getConnection(dbName, null, null);
        }
        catch(java.sql.SQLException e)
        {
            System.out.println("Error getting connection: " + e + "\n");
            System.exit(0);
        }

        System.out.println("Successful connection: " + conn + "\n");

        // Read in file data
        System.out.println("Reading the file: " + fileName);
        String inText = readFile(fileName);

        StringTokenizer strTok = new StringTokenizer(inText, ";");

        // Parse data
        while(strTok.hasMoreTokens())
        {
            String sql = strTok.nextToken().trim();
            if(!sql.equals("") && !sql.startsWith("connect"))
            {
                System.out.println("SQL = " + sql);
                try
                {
                    Vector targetStrings = new Vector();
                    String processedSql = "";
                    String VALUES = "VALUES";
                    int startIndex = -1;

                    // Locate start of any values in SQL statement
                    int valueStart = sql.toUpperCase().indexOf(VALUES);
                    if(valueStart != -1)
                    {
                        startIndex = sql.indexOf("(", valueStart + VALUES.length());
                        if(startIndex < sql.length())
                        {
                            processedSql = sql.substring(0, startIndex);
                        }
                    }

                    // Create processed SQL statement while parsing out values
                    if(startIndex != -1)
                    {
                        int index = startIndex;
                        int literalStart = -1;
                        while(index < sql.length())
                        {
                            if(sql.charAt(index) == '\'')
                            {
                                if((literalStart == -1) && ((index + 1) < sql.length()))
                                {
                                    literalStart = index +1;
                                }
                                else
                                {
                                    String targetString = sql.substring(literalStart, index);
                                    targetStrings.addElement(targetString);
                                    literalStart = -1;
                                    processedSql += "?";
                                    if((index + 1) < sql.length())
                                    {
				        index++;
                                    }
                                }
                            }
                            if((literalStart == -1) && (sql.charAt(index) == ')'))
                            {
                                break;
                            }
                            else if(literalStart == -1)
                            {
                                processedSql += sql.charAt(index);
                            }

                            index++;
                        }
                        processedSql += ")";

                        if(conn != null)
                        {
                            // Execute processed SQL statement with parsed values
                            PreparedStatement stmt = conn.prepareStatement(processedSql);
                            for(int loop=0; loop < targetStrings.size(); loop++)
                            {
                                String arg = (String)targetStrings.elementAt(loop);
                                stmt.setString((loop + 1), arg);
                            }
                            int resultCount = stmt.executeUpdate();
                            stmt.close();
                            System.out.println(resultCount + " row(s) updated.\n");
                        }
                    }
                    else
                    {
                        if(conn != null)
                        {
                            // Execute SQL statement
                            PreparedStatement stmt = conn.prepareStatement(sql);
                            int resultCount = stmt.executeUpdate();
                            stmt.close();
                            System.out.println(resultCount + " row(s) updated.\n");
                        }
                    }
                }
                catch(SQLException e)
                {
                    // Show SQL error, but continue processing anyway
                    System.out.println(e + "\n");
                }
            }
        }
    }

    public static String readFile(String fileName)
    {
        FileInputStream inStream;
        int fileSize = 0;
        byte[] inData = null;
        String readData = new String("");

        // Read in file data
        try
        {
            File theFile = new File(fileName);
            inStream = new FileInputStream(fileName);
            fileSize = (int)(theFile.length());
            inData = new byte[fileSize];
            inStream.read(inData, 0, fileSize);
            inStream.close();
        }
        catch(Exception e)
        {
            System.out.print ("Error reading file: " + e + "\n");
            System.exit(0);
        }

        // Convert the data
        try
        {
            readData = new String(inData, 0, fileSize, "ASCII");
        }
        catch(UnsupportedEncodingException e)
        {
            System.out.println("Read conversion error: " + e + "\n");
            System.exit(0);
        }

        return readData;
    }
}
